;\a86v3\ atest.asm...
; PRINT A FLOATING POINT VALUE, STORED IN: MEM32
; PRINT AN INTEGER VALUE STORED IN: MEM16
; --------------------------------------------------
  jmp  START
; --------------------------------------------------
; Variables
; --------------------------------------------------
mem16      dd  12345678
mem32      dq  564.33
; --------------------------------------------------
sw87       dw  0            ; x87 status word
integer    dd  0
fraction   dd  0
decbuff    db  25 dup(0)
decsign    db  0
x87buff    dq  0
LO_word    DW  0            ; Low Order word
HO_word    DW  0            ; High Order word
; --------------------------------------------------
C10        DW  10           ; constant for division
C10000     DW  10000        ; constant for division
IntString  DB  '          ','$',' '  ; integer string
; --------------------------------------------------
; Equates
; --------------------------------------------------
cr	equ	13	; return
lf	equ	10	; linefeed
; --------------------------------------------------
; --------------------------------------------------
START   PROC    near
; --------------------------------------------------
; --------------------------------------------------
;###############################################
; PRINT A FLOATING POINT VALUE, STORED IN: MEM32
; --------------------------------------------------
  FLD mem32		; load floating point number
  FST x87buff		; store in print buffer: Quad
  call GET_SIGN
  lea si, mem32         ; point SI to 32 bit operand
  call FLOAT2ASC
  mov dx, offset (decbuff)
  call PRINTSTR
  call NEWLINE
;########################################
; PRINT AN INTEGER VALUE STORED IN: MEM16
; --------------------------------------------------
  mov  si, offset(mem16)
  mov  ax, [si]
  mov  dx, [si+2]
  mov cx, 10            ; string length
  lea si, IntString     ; point to string
  call BINASC
  call FIXINT
  mov dx, offset (IntString)
  call PRINTSTR
  call NEWLINE
; --------------------------------------------------
; --------------------------------------------------
Done:
  INT 20H
; --------------------------------------------------
START   ENDP
; --------------------------------------------------
; --------------------------------------------------
GET_SIGN  PROC  NEAR
  mov decsign, 0        ; clear sign-flag
  FLDZ			; load a zero
  FCOM			; compare st(1) to st(0)
  FSTSW sw87            ; store x87 status word
  FWAIT
  mov ax, sw87		; copy status register
  and ax, 4700h         ; 0100-0111-0000-0000b
  jnz SIGN_POSITIVE
  mov decsign, 1        ; set negative sign-flag
;
SIGN_POSITIVE:
  FINIT
  ret
GET_SIGN  ENDP
; --------------------------------------------------
; --------------------------------------------------
; Convert a 32 bit floating point number into an ascii string.
; Enter: with SI pointing to 32 bit operand.
; Exit : 'decbuff' contains external ascii string.
; --------------------------------------------------
FLOAT2ASC  PROC  NEAR
  xor ax, ax		; zero AX register
  mov al, decsign	; load sign flag
  cmp ax, 0		; 0=positive, 1=negative
  jz IS_POSVAL
  FLD mem32             ; load floating point number
  FCHS                  ; change sign to positive
  FSTP mem32            ; store as positive number
;
IS_POSVAL:
  clc                   ; EXTRACT WHOLE NUMBER:
  FILD C10000           ; load multiplier
  FLD mem32             ; load floating point number
  FLD mem32             ; twice
  FRNDINT               ; round to integer
  FXCH                  ; exchange st(0) with st(1)
  FCOM                  ; did number round upward ?
  FSTSW sw87            ; store x87 status word
  FWAIT
  and sw87, 100h        ; strip away excess bits
  cmp sw87, 100h        ; is H.O. '1' bit set
  jne NOT_GREATER
  FXCH                  ; exchange st(0) with st(1)
  FLD1                  ; load '1'
  FSUBP st(1)           ; decrement st(1)
  jmp RESUME_CONV
;
NOT_GREATER:
  FXCH                  ; exchange st(0) with st(1)
RESUME_CONV:
  FIST integer          ; save integer value
  FSUBP st(1)           ; subtract integer from float
  FMUL  st(1)           ; multiply x 10,000
  FMULP st(1)           ; multiply x 10,000 and pop
  FRNDINT               ; round to integer
  FISTP fraction        ; store fraction as integer
;
DO_INTEGER:
  mov ax, W integer     ; load L.O. word
  mov dx, W integer+2   ; load H.O. word
  mov cx, 10            ; string length
  lea si, IntString     ; point to string
  call BINASC
  call FIXINT
  call LOAD_DECBUF1
  push di               ; save decbuff pointer
;
DO_FRACTION:
  mov ax, W fraction    ; load L.O. word
  mov dx, W fraction+2  ; load H.O. word
  mov cx, 10            ; string length
  lea si, IntString     ; point to string
  call BINASC
  call FIXINT
  pop di                ; point DI to decbuff
  call LOAD_DECBUF2
  mov [di], '$'         ; add termination character
  dec di                ; push DI back to last character
  mov al, B [di]        ; last embedded character
  cmp al, '0'           ; is it an ascii '0'
  jz FRAC_END
  mov al, [di-1]        ; previous character
  cmp al, '0'           ; is this an ascii '0'
  jnz FRAC_END
  mov B [di], '0'       ; make final character a '0'
;
FRAC_END:
  ret
FLOAT2ASC ENDP
; --------------------------------------------------
; --------------------------------------------------
LOAD_DECBUF2  PROC  NEAR
  lea si, IntString     ; address of integer string
  mov cx, si            ; copy source address to CX
  add cx, 10            ; point to end of string
;
LOADER1:
  inc di                ; increment destination
  mov al, [si]          ; get ascii number
  cmp al, '$'           ; at the end yet ?
  jz DONE_DECBUF
  mov [di], al          ; store ascii number
  inc si                ; increment source pointer
  cmp si, cx            ; is SI at end ?
  jz DONE_DECBUF
  jmp LOADER1           ; repeat
;
DONE_DECBUF:
  ret
LOAD_DECBUF2  ENDP
; --------------------------------------------------
; --------------------------------------------------
LOAD_DECBUF1  PROC  NEAR
  lea di, decbuff       ; point to decimal buffer
  push di               ; save address
  mov cx, 25            ; length of decimal buffer
CLR_BFFR:
  mov B [di], 0         ; zero buffer
  inc di                ; next byte
  loop CLR_BFFR
  pop di                ; reload buffer address
  lea si, IntString     ; address of integer string
  mov cx, si            ; copy address to CX
  add cx, 10            ; point to end of string
  cmp decsign, 0        ; test decsign for positive
  jz  POS_SIGN1
  mov B [di], '-'       ; make negative
  jmp POS_SIGN2
POS_SIGN1:
  mov B [di], ' '       ; insert space
POS_SIGN2:
  inc di                ; point to next character
  mov al, [si]          ; get ascii number
  cmp al, '$'           ; at the end yet ?
  jz SET_DECIMAL
  mov [di], al          ; store ascii number
  inc si                ; increment source pointer
  cmp si, cx            ; is SI at end ?
  jz SET_DECIMAL
  jmp POS_SIGN2         ; repeat
;
SET_DECIMAL:
  mov B [di], '.'       ; insert decimal
  ret
LOAD_DECBUF1  ENDP
; --------------------------------------------------
; --------------------------------------------------
; converts a binary number in dx:ax to printable form
; and places it in a field pointed to by si with the
; field width in cx. leading zeros are suppressed.
; --------------------------------------------------
BINASC  PROC  NEAR
  mov   di, si            ; save start of string
;
BA1:
  mov BYTE PTR [si], ' '  ; fill character
  inc   si                ; point to next field position
  loop  BA1               ; loop until done
  div   C10000            ; divide by 10,000
  mov   bx, ax            ; save quotient
  mov   ax, dx            ; move remainder back to ax
;
BA2:
  mov   cx, 4             ; number of digits to print
;
BA3:
  xor   dx, dx            ; clear High Order Word
  div   C10               ; divide by ten
  add   dl, '0'           ; convert to ascii digit
  dec   si                ; step backwards thru buffer
  cmp   si, di            ; out of space ?
  jb    BAX               ; yes, quit
  mov   [si], dl          ; store digit
  or    ax, ax            ; all digits printed ?
  jnz   BA4               ; no, keep on going
  or    bx, bx            ; any more work ?
  jz    BAX               ; no, can quit
;
BA4:
  loop  BA3               ; next digit
;
BA5:
  or    bx, bx            ; more work to do ?
  jz    BAX               ; no, can quit
  mov   ax, bx            ; get next 4 digits
  xor   bx, bx            ; show no more digits
  jmp   BA2               ; keep on going
;
BAX:
  ret
;
BINASC  ENDP
; --------------------------------------------------
; --------------------------------------------------
FIXINT  PROC  NEAR
  mov  ax, offset(IntString)
  mov  di, ax
  mov  si, ax
  mov  cl, 10
Fixint1:
  cmp  byte ptr [si], ' '  ; is it a space
  jnz  Fixint2             ; if not, exit loop
  inc  si
  loop Fixint1
  jmp  FixEnd
Fixint2:
  mov  al, byte ptr [si]
  mov  [di], al
  inc  si
  inc  di
  loop Fixint2
  mov B [di], '$'
FixEnd:
  ret
FIXINT  ENDP
; --------------------------------------------------
; --------------------------------------------------
PRINTSTR  PROC  NEAR
  push ax
  mov  ah, 9
  INT  21H
  pop  ax
  ret
PRINTSTR  ENDP
; --------------------------------------------------
; --------------------------------------------------
NEWLINE  PROC  NEAR
  mov  al, cr		; return
  call STDOUT
  mov  al, lf		; linefeed
  call STDOUT
  ret
NEWLINE  ENDP
; --------------------------------------------------
; --------------------------------------------------
; Routine for Standard Output
STDOUT  PROC  NEAR
; ascii char in al
  push dx		; save register
  mov  dl, al		; in DL for DOS call
  mov  ah, 2		; standard output
  INT  21H
  pop  dx
  ret
STDOUT  ENDP
; --------------------------------------------------
; --------------------------------------------------
ZSEG:
; --------------------------------------------------
